# -*- coding: utf-8 -*-

# Copyright (c) 2009 Efrim <efrim@azylum.org> {{{
#
#  This file is part of MPD_sima
#
#  MPD_sima is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation; either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public
#  License along with this program.
#  If not, see <http://www.gnu.org/licenses/>.
#
#
#  }}}

import sys
from optparse import (OptionParser, OptionValueError, SUPPRESS_HELP)

from os import (getpid, environ)
from os import (access, W_OK, F_OK)
from sys import (exit)

USAGE = u"""USAGE:  %prog [--help] [options]"""
DESCRIPTION = u"""
MPD_sima automagicaly queue new tracks in MPD playlist.
All command line options are overridden by their equivalent in configuration
file.
"""


def is_file_rwable(option, opt_str, value, parser):#{{{ Callbacks
    # Check either file is read/write-able or not
    if access(value, F_OK) and not access(value, W_OK):
        raise OptionValueError('no write access to "%s"' % value)
    setattr(parser.values, option.dest, value)#}}}


def get_mpd_environ():#{{{
    """
    Retrieve MPD env. var.
    """
    # TODO: get_mpd_environ() should return actual password or None
    mpd_host_and_pass = environ.get('MPD_HOST')
    if mpd_host_and_pass:
        # If password is set:
        # mpd_host_and_pass = ['pass', 'host']
        mpd_host_and_pass = mpd_host_and_pass.split('@')
        mpd_host_and_pass.reverse()

    mpd_port = environ.get('MPD_PORT')
    return (mpd_host_and_pass, mpd_port)#}}}


def clean_dict(to_clean):#{{{
    """Remove items which values are set to None"""
    for k in to_clean.keys():
        if not to_clean.get(k):
            to_clean.pop(k)
    return to_clean#}}}


# Options list
# pop out 'sw' value before creating OptionParser object.
##{{{
OPTS = list([
    {
        'sw':['-l', '--log'],
        'type': 'string',
        'dest':'logfile',
        'action': 'callback',
        'callback': is_file_rwable,
        'help': 'file to log message to, default is stdout/stderr'},
    {
        'sw': ['-p', '--pid'],
        'type': 'string',
        'dest': 'pidfile',
        'action': 'callback',
        'callback': is_file_rwable,
        'help': 'file to save PID to, default is not to store pid'},
    {
        'sw': ['-S', '--hostname'],
        'type': 'string',
        'dest': 'host',
        'help': 'Hostname MPD in running on'},
    {
        'sw': ['-P', '--port'],
        'type': 'int',
        'dest': 'port',
        'help': 'Port MPD in listening on'},
    {
        'sw':['--purge_history'],
        'action': 'store_true',
        'dest': 'do_purge_history',
        'help': SUPPRESS_HELP}])
#}}}


class StartOpt(object):#{{{
    """Command line management.
    """

    def __init__(self, script_info, log=None):#{{{
        self.info = dict(script_info)
        self.log = log
        self.options = dict({})
        self.main()#}}}

    def use_mpd_environ(self):#{{{
        """
        Get MPD_HOST/MPD_PORT environment variable and set options accordingly.
        """
        mpd_host_pass, mpd_port = get_mpd_environ()
        if mpd_host_pass:
            self.log.debug(u'MPD_HOST set to "%s"' % mpd_host_pass[0])
            self.options['host'] = mpd_host_pass[0]
            if len(mpd_host_pass) == 2 and mpd_host_pass[1]:
                self.log.debug(u'MPD_HOST contains password.')
                self.options['password'] = mpd_host_pass[1]
        if mpd_port:
            self.log.debug(u'MPD_PORT set to "%i".'
                                  % int(environ.get('MPD_PORT')))
            self.options['port'] = environ.get('MPD_PORT') #}}}

    def declare_opts(self):#{{{
        """
        Declare options in OptionParser object.
        """
        version = "MPD_sima v%s" % self.info.get('version')
        self.parser = OptionParser(version=version,
                                   usage=USAGE,
                                   prog='mpd_sima.py',
                                   description=DESCRIPTION)

        # Add all options declare in OPTS
        for opt in OPTS:
            opt_names = opt.pop('sw')
            self.parser.add_option(*opt_names, **opt)
        #}}}

    def writepid(self, pid_file):#{{{
        """Write PID to file supply as option"""
        self.log.debug(u'Writing pid file to "%s"' %
                          unicode(pid_file, 'utf-8'))
        try:
            fd_p = open(pid_file, 'w')
            fd_p.write(str(getpid()))
            fd_p.close()
        except IOError, getopt_err:
            print (u'Error trying to write pid file to %s: %s' %
                   (unicode(pid_file, 'utf-8'), getopt_err))
            exit(1)#}}}

    def main(self):#{{{
        """
        Look for env. var and parse command line.
        """
        self.use_mpd_environ()
        self.declare_opts()
        (options, args) = self.parser.parse_args()
        if options.pidfile:
            self.writepid(options.pidfile)
        self.options.update(clean_dict(options.__dict__))
        #}}}
#}}}

#
if __name__ == '__main__':
    sys.exit(0)

# VIM MODLINE
# vim: ai ts=4 sw=4 sts=4 expandtab
